package com.jiuqi.util;

import java.util.ArrayList;

public class Matrix
{
    private int colCount;
    private int rowCount;
    private ArrayList rowList;
    
    public Matrix() {
        this.colCount = 0;
        this.rowCount = 0;
        this.rowList = new ArrayList(0);
    }
    
    public Matrix(final int colCount, final int rowCount) {
        this.colCount = 0;
        this.rowCount = 0;
        this.rowList = new ArrayList(0);
        this.reset(colCount, rowCount);
    }
    
    public void clear() {
        this.rowList.clear();
        this.colCount = 0;
        this.rowCount = 0;
    }
    
    public void reset(final int newColCount, final int newRowCount) {
        this.clear();
        this.setColCount(newColCount);
        this.setRowCount(newRowCount);
    }
    
    public final int getColCount() {
        return this.colCount;
    }
    
    public final void setColCount(final int newColCount) {
        if (this.colCount == newColCount) {
            return;
        }
        final int count = (this.colCount <= newColCount) ? this.colCount : newColCount;
        for (int i = 0; i < this.rowCount; ++i) {
            final Object[] oldRow = this.rowList.get(i);
            if (oldRow != null) {
                final Object[] newRow = new Object[newColCount];
                for (int j = 0; j < count; ++j) {
                    newRow[j] = oldRow[j];
                }
                this.rowList.set(i, newRow);
            }
        }
        this.colCount = newColCount;
    }
    
    public final int getRowCount() {
        return this.rowCount;
    }
    
    public final void setRowCount(final int newRowCount) {
        if (this.rowCount == newRowCount) {
            return;
        }
        for (int i = this.rowCount - 1; i >= newRowCount; --i) {
            this.rowList.remove(i);
        }
        for (int i = this.rowCount; i < newRowCount; ++i) {
            this.rowList.add(null);
        }
        this.rowCount = newRowCount;
    }
    
    public final Object getItem(final int col, final int row) {
        final Object[] aRow = this.rowList.get(row);
        return (aRow == null) ? null : aRow[col];
    }
    
    public final void setItem(final int col, final int row, final Object value) {
        Object[] aRow = this.rowList.get(row);
        if (aRow == null) {
            if (value == null) {
                return;
            }
            aRow = new Object[this.colCount];
            this.rowList.set(row, aRow);
        }
        aRow[col] = value;
    }
    
    public final void insertCol(final int index) {
        for (int i = 0; i < this.rowCount; ++i) {
            final Object[] oldRow = this.rowList.get(i);
            if (oldRow != null) {
                final Object[] newRow = new Object[this.colCount + 1];
                for (int j = 0; j < index; ++j) {
                    newRow[j] = oldRow[j];
                }
                for (int j = index; j < this.colCount; ++j) {
                    newRow[j + 1] = oldRow[j];
                }
                this.rowList.set(i, newRow);
            }
        }
        ++this.colCount;
    }
    
    public final void insertCol(final int index, final int count) {
        for (int i = 0; i < this.rowCount; ++i) {
            final Object[] oldRow = this.rowList.get(i);
            if (oldRow != null) {
                final Object[] newRow = new Object[this.colCount + count];
                for (int j = 0; j < index; ++j) {
                    newRow[j] = oldRow[j];
                }
                for (int j = index; j < this.colCount; ++j) {
                    newRow[j + count] = oldRow[j];
                }
                this.rowList.set(i, newRow);
            }
        }
        this.colCount += count;
    }
    
    public final void insertCol(final int index, final int count, final Object[] rowItems) {
        this.insertCol(index, count);
        if (rowItems == null || rowItems.length == 0) {
            return;
        }
        final int len = Math.min(this.rowCount, rowItems.length);
        if (len == 0) {
            return;
        }
        for (int i = 0; i < count; ++i) {
            for (int row = 0; row < len; ++row) {
                this.setItem(index + i, row, rowItems[row]);
            }
        }
    }
    
    public final void duplicateCol(final int index, final int count) {
        for (int i = 0; i < this.rowCount; ++i) {
            final Object[] oldRow = this.rowList.get(i);
            if (oldRow != null) {
                final Object[] newRow = new Object[this.colCount + count];
                for (int j = 0; j <= index; ++j) {
                    newRow[j] = oldRow[j];
                }
                for (int j = 1; j <= count; ++j) {
                    newRow[index + j] = oldRow[index];
                }
                for (int j = index + 1; j < this.colCount; ++j) {
                    newRow[j + count] = oldRow[j];
                }
                this.rowList.set(i, newRow);
            }
        }
        this.colCount += count;
    }
    
    public final void deleteCol(final int index) {
        for (int i = 0; i < this.rowCount; ++i) {
            final Object[] oldRow = this.rowList.get(i);
            if (oldRow != null) {
                final Object[] newRow = new Object[this.colCount - 1];
                for (int j = 0; j < index; ++j) {
                    newRow[j] = oldRow[j];
                }
                for (int j = index + 1; j < this.colCount; ++j) {
                    newRow[j - 1] = oldRow[j];
                }
                this.rowList.set(i, newRow);
            }
        }
        --this.colCount;
    }
    
    public final void deleteCol(final int index, final int count) {
        final int c = (index + count > this.colCount) ? (this.colCount - index) : count;
        for (int i = 0; i < this.rowCount; ++i) {
            final Object[] oldRow = this.rowList.get(i);
            if (oldRow != null) {
                final Object[] newRow = new Object[this.colCount - 1];
                for (int j = 0; j < index; ++j) {
                    newRow[j] = oldRow[j];
                }
                for (int j = index + c; j < this.colCount; ++j) {
                    newRow[j - c] = oldRow[j];
                }
                this.rowList.set(i, newRow);
            }
        }
        this.colCount -= c;
    }
    
    public final void exchangeCol(final int index1, final int index2) {
        for (int i = 0; i < this.rowCount; ++i) {
            final Object[] aRow = this.rowList.get(i);
            final Object T = aRow[index1];
            aRow[index1] = aRow[index2];
            aRow[index2] = T;
        }
    }
    
    public final void moveCol(final int index1, final int index2) {
        if (index1 == index2) {
            return;
        }
        for (int i = 0; i < this.rowCount; ++i) {
            final Object[] aRow = this.rowList.get(i);
            if (aRow != null) {
                final Object T = aRow[index1];
                if (index1 > index2) {
                    for (int j = index1 - 1; j >= index2; --j) {
                        aRow[j + 1] = aRow[j];
                    }
                }
                else if (index1 < index2) {
                    for (int j = index1 + 1; j < index2; ++j) {
                        aRow[j - 1] = aRow[j];
                    }
                }
                aRow[index2] = T;
            }
        }
    }
    
    public final void insertRow(final int index) {
        this.rowList.add(index, null);
        ++this.rowCount;
    }
    
    public final void insertRow(final int index, final int count) {
        for (int i = 0; i < count; ++i) {
            this.rowList.add(index, null);
        }
        this.rowCount += count;
    }
    
    public final void insertRow(final int index, final int count, final Object[] colItems) {
        if (colItems == null || colItems.length == 0) {
            this.insertRow(index, count);
            return;
        }
        final int len = Math.min(this.colCount, colItems.length);
        for (int i = 0; i < count; ++i) {
            final Object[] newRow = new Object[this.colCount];
            for (int j = 0; j < len; ++j) {
                newRow[j] = colItems[j];
            }
            this.rowList.add(index, newRow);
        }
        this.rowCount += count;
    }
    
    public final void duplicateRow(final int index, final int count) {
        for (int i = 0; i < count; ++i) {
            final Object[] oldRow = this.rowList.get(index);
            Object[] newRow = null;
            if (oldRow != null) {
                newRow = new Object[this.colCount];
                for (int j = 0; j < this.colCount; ++j) {
                    newRow[j] = oldRow[j];
                }
            }
            this.rowList.add(index + 1, newRow);
        }
        this.rowCount += count;
    }
    
    public final void deleteRow(final int index) {
        this.rowList.remove(index);
        --this.rowCount;
    }
    
    public final void deleteRow(final int index, final int count) {
        final int c = (index + count > this.rowCount) ? (this.rowCount - index) : count;
        for (int row = index + c - 1; row >= index; --row) {
            this.rowList.remove(row);
        }
        this.rowCount -= c;
    }
    
    public final void exchangeRow(final int index1, final int index2) {
        final Object[] aRow = this.rowList.get(index1);
        this.rowList.set(index1, this.rowList.get(index2));
        this.rowList.set(index2, aRow);
    }
    
    public final void moveRow(final int index1, final int index2) {
        final Object[] aRow = this.rowList.get(index1);
        if (index1 > index2) {
            for (int i = index1 - 1; i >= index2; --i) {
                this.rowList.set(i + 1, this.rowList.get(i));
            }
        }
        else if (index1 < index2) {
            for (int i = index1 + 1; i < index2; ++i) {
                this.rowList.set(i - 1, this.rowList.get(i));
            }
        }
        this.rowList.set(index2, aRow);
    }
}
